(***********************************************************************)
(*  v      *   The Coq Proof Assistant  /  The Coq Development Team    *)
(* <O___,, *        INRIA-Rocquencourt  &  LRI-CNRS-Orsay              *)
(*   \VV/  *************************************************************)
(*    //   *      This file is distributed under the terms of the      *)
(*         *       GNU Lesser General Public License Version 2.1       *)
(***********************************************************************)

(* Finite sets library.  
 * Authors: Pierre Letouzey and Jean-Christophe Filliâtre 
 * Institution: LRI, CNRS UMR 8623 - Université Paris Sud
 *              91405 Orsay, France *)

(* $Id: extract.v,v 1.3 2003/07/08 13:43:36 filliatr Exp $ *)

(* extraction *)

Require FSet.
Require ZArith.

Extract Inductive bool => "bool" ["true" "false"].
Extract Inductive sumbool => "bool" ["true" "false"].
Extract Inductive option => "option" ["Some" "None"].
Extract Inductive sumor => "option" ["Some" "None"].
Extraction Inline Wf_nat.lt_wf_rec.
Extraction Inline Z_lt_rec.
Extraction Inline Acc_iter_2 well_founded_induction_type_2.

Module Nat : OrderedType.

  Definition t := nat.

  Definition eq := (eq nat).
  Definition lt := lt.

  Lemma eq_refl : (x:t) (eq x x). 
  Proof. Unfold eq; Auto. Save.

  Lemma eq_sym : (x,y:t) (eq x y) -> (eq y x).
  Proof. Unfold eq; Auto. Save.

  Lemma eq_trans : (x,y,z:t) (eq x y) -> (eq y z) -> (eq x z).
  Proof. Unfold eq; Intros; Rewrite H; Auto. Save.

  Lemma lt_trans : (x,y,z:t) (lt x y) -> (lt y z) -> (lt x z).
  Proof. Unfold lt; Intros; Apply Lt.lt_trans with y; Auto. Save.

  Lemma lt_not_eq : (x,y:t) (lt x y) -> ~(eq x y).
  Proof. Unfold lt eq; Intros; Omega. Save.

  Definition compare : (x,y:t)(Compare lt eq x y).
  Proof.
    Intros; Case (Compare_dec.lt_eq_lt_dec x y).
    Destruct 1; Intro.
    Constructor 1; Auto.
    Constructor 2; Auto.
    Intro; Constructor 3; Auto.
  Save.

End Nat.

Module NatRBT := FSetRBT.Make(Nat).

Extraction "rbt.ml" NatRBT.empty.

Module NatAVL := FSetAVL.Make(Nat).

Extraction "avl.ml" NatAVL.empty.
